'use strict';

var React = require('react');
var css = require('@emotion/css');
var coreReactConstant = require('@yozora/core-react-constant');
var PropTypes = require('prop-types');
var equal = require('@guanghechen/equal');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var React__default = /*#__PURE__*/_interopDefault(React);
var PropTypes__default = /*#__PURE__*/_interopDefault(PropTypes);

exports.TexLang = void 0;
(function (TexLang) {
    TexLang["MML"] = "MathML";
    TexLang["TEX"] = "TeX";
})(exports.TexLang || (exports.TexLang = {}));

const initialMathJaxContext = {
    MathJax3: null,
    language: exports.TexLang.TEX,
};
const MathJaxContextType = React__default.default.createContext(initialMathJaxContext);

class MathError extends React__default.default.PureComponent {
    static displayName = 'MathError';
    static propTypes = {
        lang: PropTypes__default.default.oneOf([exports.TexLang.MML, exports.TexLang.TEX]).isRequired,
        formula: PropTypes__default.default.string.isRequired,
        inline: PropTypes__default.default.bool.isRequired,
        error: PropTypes__default.default.any.isRequired,
    };
    render() {
        const { lang, formula, inline, error } = this.props;
        if (inline) {
            return (React__default.default.createElement("span", { className: classes.error },
                React__default.default.createElement("span", { className: classes.errorDetails }, error),
                React__default.default.createElement("span", { className: classes.meta },
                    "[lang: ",
                    lang,
                    ", inline: ",
                    String(inline),
                    "]"),
                React__default.default.createElement("code", { className: classes.source }, formula)));
        }
        return (React__default.default.createElement("div", { className: classes.error },
            React__default.default.createElement("div", { className: classes.errorDetails }, error),
            React__default.default.createElement("div", { className: classes.meta },
                "[lang: ",
                lang,
                ", inline: ",
                String(inline),
                "]"),
            React__default.default.createElement("pre", { className: classes.source }, formula)));
    }
}
const vars = {
    border: `1px solid var(${coreReactConstant.TokenNames.colorBorderError}, red)`,
    errorColor: `var(${coreReactConstant.TokenNames.colorBorderError}, red)`,
    metaColor: `var(${coreReactConstant.TokenNames.colorCodeTitle}, '#333')`,
    sourceColor: `var(${coreReactConstant.TokenNames.colorMath}, 'blue')`,
    fontFamilyCode: `var(${coreReactConstant.CommonTokenNames.fontFamilyCode}, Consolas, 'Source Code Pro', 'Roboto Mono', monospace, sans-serif)`,
    fontSizeCode: `var(${coreReactConstant.CommonTokenNames.fontSizeCode}, 1rem)`,
};
const classes = {
    error: css.css({
        display: 'block',
        width: '100%',
        height: '100%',
        color: vars.errorColor,
        fontFamily: vars.fontFamilyCode,
        fontWeight: 'normal',
        fontSize: '16px',
        border: vars.border,
    }),
    errorDetails: css.css({
        display: 'block',
        width: '100%',
        minHeight: '100%',
        padding: '0.5rem',
        background: 'transparent',
        whiteSpace: 'pre-wrap',
        textAlign: 'left',
    }),
    meta: css.css({
        padding: 8,
        margin: 0,
        color: vars.metaColor,
    }),
    source: css.css({
        overflow: 'auto',
        padding: 8,
        margin: 0,
        color: vars.sourceColor,
        fontFamily: vars.fontFamilyCode,
        fontSize: vars.fontSizeCode,
    }),
};

class MathJaxNodeWithoutContext extends React__default.default.Component {
    static displayName = 'MathJaxNodeWithoutContext';
    static propTypes = {
        MathJax3: PropTypes__default.default.object.isRequired,
        language: PropTypes__default.default.string.isRequired,
        formula: PropTypes__default.default.node.isRequired,
        inline: PropTypes__default.default.bool.isRequired,
        className: PropTypes__default.default.string,
        style: PropTypes__default.default.object,
    };
    _nodeRef;
    _typesettingRef;
    constructor(props) {
        super(props);
        this._nodeRef = { current: null };
        this._typesettingRef = { current: false };
        this.state = {
            error: undefined,
        };
    }
    shouldComponentUpdate(nextProps, nextState) {
        const props = this.props;
        const state = this.state;
        return (state.error !== nextState.error ||
            props.MathJax3 !== nextProps.MathJax3 ||
            props.language !== nextProps.language ||
            props.formula !== nextProps.formula ||
            props.inline !== nextProps.inline ||
            props.className !== nextProps.className ||
            !equal.isEqual(props.style, nextProps.style));
    }
    render() {
        const { formula, inline, className, style } = this.props;
        const { error } = this.state;
        if (typeof error === 'string') {
            return (React__default.default.createElement(MathError, { lang: this.props.language, formula: this.props.formula, inline: inline, error: error }));
        }
        return React__default.default.createElement(inline ? 'span' : 'div', {
            ref: this._nodeRef,
            className,
            style,
        }, inline ? `$${formula}$` : `$$${formula.replace(/\n/g, ' ')}$$`);
    }
    componentDidMount() {
        this._typeset();
    }
    componentDidUpdate(prevProps) {
        const props = this.props;
        if (props.formula !== prevProps.formula ||
            props.inline !== prevProps.inline ||
            props.MathJax3 !== prevProps.MathJax3) {
            this._typeset();
        }
    }
    componentWillUnmount() {
        this._onTypesetDone();
    }
    _typeset() {
        const { MathJax3 } = this.props;
        if (!MathJax3) {
            throw new Error("Could not find MathJax3 while attempting typeset! Probably MathJax3 script hasn't been loaded or MathJaxContextType.Provider is not in the hierarchy");
        }
        const node = this._nodeRef.current;
        if (node) {
            if (!this._typesettingRef.current) {
                this._typesettingRef.current = true;
                void MathJax3.startup.promise
                    .then(() => {
                    MathJax3.typesetClear([node]);
                    return MathJax3.typesetPromise([[node]]);
                })
                    .then(() => {
                    this.setState({ error: undefined });
                    this._onTypesetDone();
                })
                    .catch(err => {
                    console.log('err:', err);
                    this.setState({ error: `Typesetting failed: ${err.message ?? err.toString()}` });
                    this._onTypesetDone();
                });
            }
        }
    }
    _onTypesetDone() {
        this._typesettingRef.current = false;
    }
}

const MathJaxNode = props => {
    const { inline = false, className, style } = props;
    const { MathJax3, language } = React__default.default.useContext(MathJaxContextType);
    const formula = props.formula.trim();
    if (MathJax3) {
        return (React__default.default.createElement(MathJaxNodeWithoutContext, { MathJax3: MathJax3, language: language, formula: formula, inline: inline, className: className, style: style }));
    }
    return inline ? (React__default.default.createElement("span", { className: className, style: style },
        "$",
        formula,
        "$")) : (React__default.default.createElement("div", { className: className, style: style },
        "$$",
        formula,
        "$$"));
};

let MathJax3Promise = null;
function loadMathJax3(mathjax3Src, mathjax3Config) {
    if (MathJax3Promise === null) {
        MathJax3Promise = new Promise((resolve, reject) => {
            if (typeof window === 'undefined') {
                resolve(null);
                return;
            }
            const w = window;
            if (mathjax3Config && !w.MathJax)
                w.MathJax = mathjax3Config;
            const script = document.createElement('script');
            script.type = 'text/javascript';
            script.src = mathjax3Src;
            script.async = false;
            script.addEventListener('load', () => {
                const mathjax = w.MathJax;
                resolve(mathjax);
            });
            script.addEventListener('error', err => {
                reject(err);
            });
            const head = document.getElementsByTagName('head');
            head?.[0]?.appendChild?.(script);
        });
    }
    return MathJax3Promise;
}

class MathJaxProvider extends React__default.default.Component {
    static displayName = 'MathJaxProvider';
    static propTypes = {
        mathjaxSrc: PropTypes__default.default.string,
        mathjaxConfig: PropTypes__default.default.object,
        loading: PropTypes__default.default.node,
        children: PropTypes__default.default.node,
        onLoad: PropTypes__default.default.func,
        onError: PropTypes__default.default.func,
    };
    _cancelLoad;
    constructor(props) {
        super(props);
        this._cancelLoad = undefined;
        this.state = {
            loaded: false,
            context: initialMathJaxContext,
        };
    }
    shouldComponentUpdate(nextProps, nextState) {
        const props = this.props;
        const state = this.state;
        return (state.loaded !== nextState.loaded ||
            !equal.isEqual(state.context, nextState.context) ||
            props.loading !== nextProps.loading ||
            props.children !== nextProps.children ||
            props.mathjaxSrc !== nextProps.mathjaxSrc ||
            !equal.isEqual(props.mathjaxConfig, nextProps.mathjaxConfig));
    }
    render() {
        const { loading = null, children } = this.props;
        const { loaded, context } = this.state;
        if (!loaded && loading)
            return React__default.default.createElement(React__default.default.Fragment, null, loading);
        return React__default.default.createElement(MathJaxContextType.Provider, { value: context }, children);
    }
    componentDidMount() {
        void this.load();
    }
    componentDidUpdate(prevProps) {
        const props = this.props;
        if (props.mathjaxSrc !== prevProps.mathjaxSrc ||
            !equal.isEqual(props.mathjaxConfig, prevProps.mathjaxConfig)) {
            void this.load();
        }
    }
    componentWillUnmount() {
        const { MathJax3 } = this.state.context;
        void this.clear().then(() => {
            MathJax3?.texReset();
        });
    }
    async load() {
        await this.clear();
        const { mathjaxSrc = 'https://cdn.jsdelivr.net/npm/mathjax@3.2.2/es5/tex-mml-chtml.js', mathjaxConfig = defaultMathjaxConfig, } = this.props;
        let cancelled = false;
        const loadResult = loadMathJax3(mathjaxSrc, mathjaxConfig).then((MathJax3) => {
            if (cancelled)
                return () => { };
            this.setState(prevState => ({
                loaded: true,
                context: { ...prevState.context, MathJax3: MathJax3 },
            }));
            return () => { };
        });
        this._cancelLoad = async () => {
            cancelled = true;
            const unregister = await loadResult;
            unregister();
        };
    }
    async clear() {
        if (this._cancelLoad) {
            await this._cancelLoad();
            this._cancelLoad = undefined;
        }
    }
    onLoad = (MathJax3) => {
        this.props.onLoad?.(MathJax3);
    };
    onError = (MathJax3, error) => {
        this.props.onError?.(MathJax3, error);
    };
}
const defaultMathjaxConfig = {
    loader: {
        load: [
            '[tex]/ams',
            '[tex]/color',
            '[tex]/colortbl',
            '[tex]/tagformat',
            '[tex]/unicode',
            'input/tex',
            'output/chtml',
        ],
    },
    tex: {
        tags: 'ams',
        packages: {
            '[+]': ['base', 'ams', 'color', 'colortbl', 'tagformat', 'unicode'],
        },
        inlineMath: [['$', '$']],
        displayMath: [['$$', '$$']],
        processEnvironments: true,
        processRefs: true,
    },
    tagformat: {
        number: (n) => n.toString(),
        tag: (tag) => '(' + tag + ')',
        id: (id) => 'mjx-eqn:' + id.replace(/\s/g, '_'),
        url: (id, base) => base + '#' + encodeURIComponent(id),
    },
    svg: {
        fontCache: 'global',
    },
    options: {
        skipHtmlTags: ['noscript', 'style', 'textarea', 'pre', 'code'],
        ignoreHtmlClass: 'tex2jax_ignore',
        renderActions: {
            addMenu: [],
        },
    },
};

exports.MathError = MathError;
exports.MathJaxContextType = MathJaxContextType;
exports.MathJaxNode = MathJaxNode;
exports.MathJaxNodeWithoutContext = MathJaxNodeWithoutContext;
exports.MathJaxProvider = MathJaxProvider;
exports.classes = classes;
exports.initialMathJaxContext = initialMathJaxContext;
exports.loadMathJax3 = loadMathJax3;
